@@ -24,6 +24,18 @@ makedocs(
     sitename = "Documenter.jl",
     authors = "Michael Hatherly, Morten Piibeleht, and contributors.",
     linkcheck = !("skiplinks" in ARGS),
+    linkcheck_ignore = [
+        # timelessrepo.com seems to 404 on any CURL request...
+        "http://timelessrepo.com/json-isnt-a-javascript-subset",
+        # We'll ignore links that point to GitHub's edit pages, as they redirect to the
+        # login screen and cause a warning:
+        r"https://github.com/([A-Za-z0-9_.-]+)/([A-Za-z0-9_.-]+)/edit(.*)",
+    ] ∪ (get(ENV, "GITHUB_ACTIONS", nothing)  == "true" ? [
+        # Extra ones we ignore only on CI.
+        #
+        # It seems that CTAN blocks GitHub Actions?
+        "https://ctan.org/pkg/minted",
+    ] : []),
     pages = [
         "Home" => "index.md",
         "Manual" => Any[
