@@ -51,11 +51,12 @@ include("Deps.jl")
 
 import .Utilities: Selectors
 import .Writers.HTMLWriter: HTML, asset
-import .Writers.HTMLWriter.RD: KaTeX, MathJax
+import .Writers.HTMLWriter.RD: KaTeX, MathJax, MathJax2, MathJax3
 
 # User Interface.
 # ---------------
-export Deps, makedocs, deploydocs, hide, doctest, DocMeta, KaTeX, MathJax, asset
+export Deps, makedocs, deploydocs, hide, doctest, DocMeta, asset,
+    KaTeX, MathJax, MathJax2, MathJax3
 
 """
     makedocs(
@@ -185,7 +186,7 @@ when a minor version changes (i.e. except in patch releases).
 
 **`checkdocs`** instructs [`makedocs`](@ref) to check whether all names within the modules
 defined in the `modules` keyword that have a docstring attached have the docstring also
-listed in the manual (e.g. there's a `@docs` blocks with that docstring). Possible values
+listed in the manual (e.g. there's a `@docs` block with that docstring). Possible values
 are `:all` (check all names; the default), `:exports` (check only exported names) and
 `:none` (no checks are performed). If `strict` is also enabled then the build will fail if
 any missing docstrings are encountered.
@@ -199,6 +200,9 @@ will fail if there are any broken (400+ status code) links. Default: `false`.
 be a list of strings (which get matched exactly) or `Regex` objects. By default nothing is
 ignored.
 
+**`linkcheck_timeout`** configures how long `curl` waits (in seconds) for a link request to
+return a response before giving up. The default is 10 seconds.
+
 **`strict`** -- [`makedocs`](@ref) fails the build right before rendering if it encountered
 any errors with the document in the previous build phases.
 
@@ -244,7 +248,7 @@ $(SIGNATURES)
 
 Allows a page to be hidden in the navigation menu. It will only show up if it happens to be
 the current page. The hidden page will still be present in the linear page list that can be
-accessed via the previous and next page links. The title of the hidden page can be overriden
+accessed via the previous and next page links. The title of the hidden page can be overridden
 using the `=>` operator as usual.
 
 # Usage
@@ -301,7 +305,10 @@ include("deployconfig.jl")
         make   = nothing | <Function>,
         devbranch = "master",
         devurl = "dev",
-        versions = ["stable" => "v^", "v#.#", devurl => devurl]
+        versions = ["stable" => "v^", "v#.#", devurl => devurl],
+        push_preview    = false,
+        repo_previews   = repo,
+        branch_previews = branch,
     )
 
 Converts markdown files generated by [`makedocs`](@ref) to HTML and pushes them to `repo`.
@@ -343,7 +350,7 @@ currently running environment. See the manual section about
 **`root`** has the same purpose as the `root` keyword for [`makedocs`](@ref).
 
 **`target`** is the directory, relative to `root`, where generated content that should be
-deployed to `gh-pages` is written to. written to. It should generally be the same as
+deployed to `gh-pages` is written to. It should generally be the same as
 [`makedocs`](@ref)'s `build` and defaults to `"build"`.
 
 **`branch`** is the branch where the generated documentation is pushed. If the branch does
@@ -359,10 +366,10 @@ It can be used e.g. for a Markdown build. The following example installed the `p
 deps = Deps.pip("pygments", "mkdocs")
 ```
 
-**`make`** is the function used to specify an additonal build phase. By default, nothing gets
+**`make`** is the function used to specify an additional build phase. By default, nothing gets
 executed.
 
-**`devbranch`** is the branch that "tracks" the in-development version of the  generated
+**`devbranch`** is the branch that "tracks" the in-development version of the generated
 documentation. By default this value is set to `"master"`.
 
 **`devurl`** the folder that in-development version of the docs will be deployed.
@@ -374,7 +381,7 @@ The default (`forcepush = false`) is to push a new commit, but when
 force pushed, erasing the Git history on the deployment branch.
 
 **`versions`** determines content and order of the resulting version selector in
-the generated html. The following entries are valied in the `versions` vector:
+the generated html. The following entries are valid in the `versions` vector:
  - `"v#"`: includes links to the latest documentation for each major release cycle
    (i.e. `v2.0`, `v1.1`).
  - `"v#.#"`: includes links to the latest documentation for each minor release cycle
@@ -388,7 +395,31 @@ the generated html. The following entries are valied in the `versions` vector:
    (as in e.g. `"stable" => "v^"`).
 
 **`push_preview`** a boolean that specifies if preview documentation should be
-deployed from pull requests or not.
+deployed from pull requests or not. If your published documentation is hosted
+at `"https://USER.github.io/PACKAGE.jl/stable`, by default the preview will be
+hosted at `"https://USER.github.io/PACKAGE.jl/previews/PR##"`.
+
+**`branch_previews`** is the branch to which pull request previews are deployed.
+It defaults to the value of `branch`.
+
+**`repo_previews`** is the remote repository to which pull request previews are
+deployed. It defaults to the value of `repo`.
+
+!!! note
+    Pull requests made from forks will not have previews.
+    Hosting previews requires access to the deploy key.
+    Therefore, previews are available only for pull requests that were
+    submitted directly from the main repository.
+
+# Releases vs development branches
+
+[`deploydocs`](@ref) will automatically figure out whether it is deploying the documentation
+for a tagged release or just a development branch (usually, based on the environment
+variables set by the CI system).
+
+With versioned tags, [`deploydocs`](@ref) discards the build metadata (i.e. `+` and
+everything that follows it) from the version number when determining the name of the
+directory into which the documentation gets deployed. Pre-release identifiers are preserved.
 
 # See Also
 
@@ -404,6 +435,9 @@ function deploydocs(;
         repo   = error("no 'repo' keyword provided."),
         branch = "gh-pages",
 
+        repo_previews   = repo,
+        branch_previews = branch,
+
         deps   = nothing,
         make   = nothing,
 
@@ -415,8 +449,20 @@ function deploydocs(;
         push_preview::Bool = false,
     )
 
-    subfolder = deploy_folder(deploy_config; repo=repo, devbranch=devbranch, push_preview=push_preview, devurl=devurl)
-    if subfolder !== nothing
+    deploy_decision = deploy_folder(deploy_config;
+                                    branch=branch,
+                                    branch_previews=branch_previews,
+                                    devbranch=devbranch,
+                                    devurl=devurl,
+                                    push_preview=push_preview,
+                                    repo=repo,
+                                    repo_previews=repo_previews)
+    if deploy_decision.all_ok
+        deploy_branch = deploy_decision.branch
+        deploy_repo = deploy_decision.repo
+        deploy_subfolder = deploy_decision.subfolder
+        deploy_is_preview = deploy_decision.is_preview
+
         # Add local bin path if needed.
         Deps.updatepath!()
         # Install dependencies when applicable.
@@ -447,13 +493,14 @@ function deploydocs(;
                 @debug "running extra build steps."
                 make()
             end
-            @debug "pushing new documentation to remote: '$repo:$branch'."
+            @debug "pushing new documentation to remote: '$deploy_repo:$deploy_branch'."
             mktempdir() do temp
                 git_push(
-                    root, temp, repo;
-                    branch=branch, dirname=dirname, target=target,
-                    sha=sha, deploy_config=deploy_config, subfolder=subfolder,
+                    root, temp, deploy_repo;
+                    branch=deploy_branch, dirname=dirname, target=target,
+                    sha=sha, deploy_config=deploy_config, subfolder=deploy_subfolder,
                     devurl=devurl, versions=versions, forcepush=forcepush,
+                    is_preview=deploy_is_preview,
                 )
             end
         end
@@ -474,6 +521,7 @@ function git_push(
         root, temp, repo;
         branch="gh-pages", dirname="", target="site", sha="", devurl="dev",
         versions, forcepush=false, deploy_config, subfolder,
+        is_preview::Bool = false,
     )
     dirname = isempty(dirname) ? temp : joinpath(temp, dirname)
     isdir(dirname) || mkpath(dirname)
@@ -484,8 +532,8 @@ function git_push(
     function git_commands(sshconfig=nothing)
         # Setup git.
         run(`git init`)
-        run(`git config user.name "zeptodoctor"`)
-        run(`git config user.email "44736852+zeptodoctor@users.noreply.github.com"`)
+        run(`git config user.name "Documenter.jl"`)
+        run(`git config user.email "documenter@juliadocs.github.io"`)
         if sshconfig !== nothing
             run(`git config core.sshCommand "ssh -F $(sshconfig)"`)
         end
@@ -507,8 +555,13 @@ function git_push(
         try
             run(`git checkout -b $branch upstream/$branch`)
         catch e
+            @info """
+            Checking out $branch failed, creating a new orphaned branch.
+            This usually happens when deploying to a repository for the first time and
+            the $branch branch does not exist yet. The fatal error above is expected output
+            from Git in this situation.
+            """
             @debug "checking out $branch failed with error: $e"
-            @debug "creating a new local $branch branch."
             run(`git checkout --orphan $branch`)
             run(`git commit --allow-empty -m "Initial empty commit for docs"`)
         end
@@ -554,15 +607,17 @@ function git_push(
     end
 
     if authentication_method(deploy_config) === SSH
-        # Extract host from repo as everything up to first ':' or '/' character
-        host = match(r"(.*?)[:\/]", repo)[1]
-
-        # The upstream URL to which we push new content and the ssh decryption commands.
-        upstream = "git@$(replace(repo, "$host/" => "$host:"))"
+        # Get the parts of the repo path and create upstream repo path
+        user, host, upstream = user_host_upstream(repo)
 
         keyfile = abspath(joinpath(root, ".documenter"))
         try
-            write(keyfile, base64decode(documenter_key(deploy_config)))
+            if is_preview
+                keycontent = documenter_key_previews(deploy_config)
+            else
+                keycontent = documenter_key(deploy_config)
+            end
+            write(keyfile, base64decode(keycontent))
         catch e
             @error """
             Documenter failed to decode the DOCUMENTER_KEY environment variable.
@@ -580,7 +635,7 @@ function git_push(
                 """
                 Host $host
                     StrictHostKeyChecking no
-                    User git
+                    User $user
                     HostName $host
                     IdentityFile "$keyfile"
                     IdentitiesOnly yes
@@ -623,6 +678,29 @@ function rm_and_add_symlink(target, link)
     symlink(target, link)
 end
 
+"""
+    user_host_upstream(repo)
+
+Disassemble repo address into user, host, and path to repo. If no user is given, default to
+"git". Reassemble user, host and path into an upstream to `git push` to.
+"""
+function user_host_upstream(repo)
+    #= the regex has three parts:
+    (?:([^@]*)@)?  matches any number of characters up to the first "@", if present,
+        capturing only the characters before the "@" - this captures the username
+    (?:([^\/:]*)[\/:]){1}  matches exactly one instance of any number of characters
+        other than "/" or ":" up to the first "/" or ":" - this captures the hostname
+    [\/]?(.*)  matches the rest of the repo, except an initial "/" if present (e.g. if
+        repo is of the form usr@host:/path/to/repo) - this captures the path on the host
+    =#
+    m = match(r"(?:([^@]*)@)?(?:([^\/:]*)[\/:]){1}[\/]?(.*)", repo)
+    (m === nothing) && error("Invalid repo path $repo")
+    user, host, pth = m.captures
+    user = (user === nothing) ? "git" : user
+    upstream = "$user@$host:$pth"
+    return user, host, upstream
+end
+
 """
     gitrm_copy(src, dst)
 
