@@ -490,7 +490,13 @@ end
 linerange(doc) = linerange(doc.text, doc.data[:linenumber])
 
 function linerange(text, from)
-    lines = sum([isodd(n) ? newlines(s) : 0 for (n, s) in enumerate(text)])
+    # text is assumed to be a Core.SimpleVector (svec) from the .text field of a Docs.DocStr object.
+    # Hence, we need to be careful when summing over an empty svec below.
+    #
+    # Also, the isodd logic _appears_ to be there to handle variable interpolation into docstrings. In that case,
+    # the .text field seems to become longer than just 1 element and every even element is the interpolated object,
+    # and only the odd ones actually contain the docstring text as a string.
+    lines = sum(Int[isodd(n) ? newlines(s) : 0 for (n, s) in enumerate(text)])
     return lines > 0 ? (from:(from + lines + 1)) : (from:from)
 end
 
@@ -685,6 +691,16 @@ struct Default{T}
 end
 Base.getindex(default::Default) = default.value
 
+"""
+    $(SIGNATURES)
+
+Extracts the language identifier from the info string of a Markdown code block.
+"""
+function codelang(infostring::AbstractString)
+    m = match(r"^\s*(\S*)", infostring)
+    return m[1]
+end
+
 include("DOM.jl")
 include("MDFlatten.jl")
 include("TextDiff.jl")
