@@ -7,8 +7,8 @@ A module for rendering `Document` objects to HTML.
 [`Documenter.makedocs`](@ref): `authors`, `pages`, `sitename`, `version`.
 The behavior of [`HTMLWriter`](@ref) can be further customized by setting the `format`
 keyword of [`Documenter.makedocs`](@ref) to a [`HTML`](@ref), which accepts the following
-keyword arguments: `analytics`, `assets`, `canonical`, `disable_git`, `edit_link` and
-`prettyurls`.
+keyword arguments: `analytics`, `assets`, `canonical`, `disable_git`, `edit_link`,
+`prettyurls`, `collapselevel`, `sidebar_sitename`, `highlights`, `mathengine` and `footer`.
 
 **`sitename`** is the site's title displayed in the title bar and at the top of the
 *navigation menu. This argument is mandatory for [`HTMLWriter`](@ref).
@@ -171,15 +171,15 @@ struct KaTeX <: MathEngine
 end
 
 """
-    MathJax(config::Dict = <default>, override = false)
+    MathJax2(config::Dict = <default>, override = false)
 
-An instance of the `MathJax` type can be passed to [`HTML`](@ref) via the `mathengine`
-keyword to specify that the [MathJax rendering engine](https://www.mathjax.org/) should be
+An instance of the `MathJax2` type can be passed to [`HTML`](@ref) via the `mathengine`
+keyword to specify that the [MathJax v2 rendering engine](https://www.mathjax.org/) should be
 used in the HTML output to render mathematical expressions.
 
 A dictionary can be passed via the `config` argument to configure MathJax. It gets passed to
-the [`MathJax.Hub.Config`](https://docs.mathjax.org/en/latest/options/) function. By
-default, Documenter set custom configuration for `tex2jax`, `config`, `jax`, `extensions`
+the [`MathJax.Hub.Config`](https://docs.mathjax.org/en/v2.7-latest/options/) function. By
+default, Documenter sets custom configurations for `tex2jax`, `config`, `jax`, `extensions`
 and `Tex`.
 
 By default, the user-provided dictionary gets _merged_ with the default dictionary (i.e. the
@@ -188,29 +188,70 @@ setting your own `tex2jax` value will override the default). This can be overrid
 setting `override` to `true`, in which case the default values are ignored and only the
 user-provided dictionary is used.
 """
-struct MathJax <: MathEngine
+struct MathJax2 <: MathEngine
     config :: Dict{Symbol,Any}
-    function MathJax(config::Union{Dict,Nothing} = nothing, override=false)
+    function MathJax2(config::Union{Dict,Nothing} = nothing, override=false)
         default = Dict(
-           :tex2jax => Dict(
-               "inlineMath" => [["\$","\$"], ["\\(","\\)"]],
-               "processEscapes" => true
-           ),
-           :config => ["MMLorHTML.js"],
-           :jax => [
-               "input/TeX",
-               "output/HTML-CSS",
-               "output/NativeMML"
-           ],
-           :extensions => [
-               "MathMenu.js",
-               "MathZoom.js",
-               "TeX/AMSmath.js",
-               "TeX/AMSsymbols.js",
-               "TeX/autobold.js",
-               "TeX/autoload-all.js"
-           ],
-           :TeX => Dict(:equationNumbers => Dict(:autoNumber => "AMS"))
+            :tex2jax => Dict(
+                "inlineMath" => [["\$","\$"], ["\\(","\\)"]],
+                "processEscapes" => true
+            ),
+            :config => ["MMLorHTML.js"],
+            :jax => [
+                "input/TeX",
+                "output/HTML-CSS",
+                "output/NativeMML"
+            ],
+            :extensions => [
+                "MathMenu.js",
+                "MathZoom.js",
+                "TeX/AMSmath.js",
+                "TeX/AMSsymbols.js",
+                "TeX/autobold.js",
+                "TeX/autoload-all.js"
+            ],
+            :TeX => Dict(:equationNumbers => Dict(:autoNumber => "AMS"))
+        )
+        new((config === nothing) ? default : override ? config : merge(default, config))
+    end
+end
+
+@deprecate MathJax(config::Union{Dict,Nothing} = nothing, override=false) MathJax2(config, override) false
+@doc "deprecated – Use [`MathJax2`](@ref) instead" MathJax
+
+"""
+    MathJax3(config::Dict = <default>, override = false)
+
+An instance of the `MathJax3` type can be passed to [`HTML`](@ref) via the `mathengine`
+keyword to specify that the [MathJax v3 rendering engine](https://www.mathjax.org/) should be
+used in the HTML output to render mathematical expressions.
+
+A dictionary can be passed via the `config` argument to configure MathJax. It gets passed to
+[`Window.MathJax`](https://docs.mathjax.org/en/latest/options/) function. By default,
+Documenter specifies in the key `tex` that `\$...\$` and `\\(...\\)` denote inline math, that AMS
+style tags should be used and the `base`, `ams` and `autoload` packages should be imported.
+The key `options`, by default, specifies which HTML classes to ignore and which to process
+using MathJax.
+
+By default, the user-provided dictionary gets _merged_ with the default dictionary (i.e. the
+resulting configuration dictionary will contain the values from both dictionaries, but e.g.
+setting your own `tex` value will override the default). This can be overridden by
+setting `override` to `true`, in which case the default values are ignored and only the
+user-provided dictionary is used.
+"""
+struct MathJax3 <: MathEngine
+    config :: Dict{Symbol,Any}
+    function MathJax3(config::Union{Dict,Nothing} = nothing, override=false)
+        default = Dict(
+            :tex => Dict(
+                "inlineMath" => [["\$","\$"], ["\\(","\\)"]],
+                "tags" => "ams",
+                "packages" => ["base", "ams", "autoload"],
+            ),
+            :options => Dict(
+                "ignoreHtmlClass" => "tex2jax_ignore",
+                "processHtmlClass" => "tex2jax_process",
+            )
         )
         new((config === nothing) ? default : override ? config : merge(default, config))
     end
@@ -276,13 +317,18 @@ you would set `highlights = ["llvm", "yaml"]`. Note that no verification is done
 provided language names are sane.
 
 **`mathengine`** specifies which LaTeX rendering engine will be used to render the math
-blocks. The options are either [KaTeX](https://katex.org/) (default) or
-[MathJax](https://www.mathjax.org/), enabled by passing an instance of [`KaTeX`](@ref) or
-[`MathJax`](@ref) objects, respectively. The rendering engine can further be customized by
-passing options to the [`KaTeX`](@ref) or [`MathJax`](@ref) constructors.
+blocks. The options are either [KaTeX](https://katex.org/) (default),
+[MathJax v2](https://www.mathjax.org/), or [MathJax v3](https://www.mathjax.org/), enabled by
+passing an instance of [`KaTeX`](@ref), [`MathJax2`](@ref), or
+[`MathJax3`](@ref) objects, respectively. The rendering engine can further be customized by
+passing options to the [`KaTeX`](@ref) or [`MathJax2`](@ref)/[`MathJax3`](@ref) constructors.
 
 **`lang`** can be used to specify the language tag of each HTML page. Default is `"en"`.
 
+**`footer`** can be a valid single-line markdown `String` or `nothing` and is displayed below
+the page navigation. Defaults to `"Powered by [Documenter.jl](https://github.com/JuliaDocs/Documenter.jl)
+and the [Julia Programming Language](https://julialang.org/)."`.
+
 # Default and custom assets
 
 Documenter copies all files under the source directory (e.g. `/docs/src/`) over
@@ -325,6 +371,7 @@ struct HTML <: Documenter.Writer
     sidebar_sitename :: Bool
     highlights    :: Vector{String}
     mathengine    :: Union{MathEngine,Nothing}
+    footer        :: Union{Markdown.MD, Nothing}
     lang          :: String
 
     function HTML(;
@@ -336,8 +383,9 @@ struct HTML <: Documenter.Writer
             analytics     :: String = "",
             collapselevel :: Integer = 2,
             sidebar_sitename :: Bool = true,
-            highlights :: Vector{String} = String[],
-            mathengine :: Union{MathEngine,Nothing} = KaTeX(),
+            highlights    :: Vector{String} = String[],
+            mathengine    :: Union{MathEngine,Nothing} = KaTeX(),
+            footer        :: Union{String, Nothing} = "Powered by [Documenter.jl](https://github.com/JuliaDocs/Documenter.jl) and the [Julia Programming Language](https://julialang.org/).",
             # deprecated keywords
             edit_branch   :: Union{String, Nothing, Default} = Default(nothing),
             lang          :: String = "en",
@@ -360,9 +408,15 @@ struct HTML <: Documenter.Writer
         if isa(edit_link, Symbol) && (edit_link !== :commit)
             throw(ArgumentError("Invalid symbol (:$edit_link) passed to edit_link."))
         end
+        if footer !== nothing
+            footer = Markdown.parse(footer)
+            if !(length(footer.content) == 1 && footer.content[1] isa Markdown.Paragraph)
+                throw(ArgumentError("footer must be a single-line markdown compatible string."))
+            end
+        end
         isa(edit_link, Default) && (edit_link = edit_link[])
         new(prettyurls, disable_git, edit_link, canonical, assets, analytics,
-            collapselevel, sidebar_sitename, highlights, mathengine, lang)
+            collapselevel, sidebar_sitename, highlights, mathengine, footer, lang)
     end
 end
 
@@ -370,28 +424,26 @@ end
 module RD
     using JSON
     using ....Utilities.JSDependencies: RemoteLibrary, Snippet, RequireJS, jsescape, json_jsescape
-    using ..HTMLWriter: KaTeX, MathJax
+    using ..HTMLWriter: KaTeX, MathJax, MathJax2, MathJax3
 
-    const requirejs_cdn = "https://cdnjs.cloudflare.com/ajax/libs/require.js/2.3.6/require.min.js"
-    const google_fonts = "https://fonts.googleapis.com/css?family=Lato|Roboto+Mono"
+    const requirejs_cdn = "file:///usr/share/javascript/requirejs/require.min.js"
+    const google_fonts = "file:///usr/share/doc/julia-doc/fontface.css"
     const fontawesome_version = "5.11.2"
     const fontawesome_css = [
-        "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/$(fontawesome_version)/css/fontawesome.min.css",
-        "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/$(fontawesome_version)/css/solid.min.css",
-        "https://cdnjs.cloudflare.com/ajax/libs/font-awesome/$(fontawesome_version)/css/brands.min.css",
+	"file:///usr/share/fonts-font-awesome/css/font-awesome.css",
     ]
 
-    const jquery = RemoteLibrary("jquery", "https://cdnjs.cloudflare.com/ajax/libs/jquery/3.4.1/jquery.min.js")
-    const jqueryui = RemoteLibrary("jqueryui", "https://cdnjs.cloudflare.com/ajax/libs/jqueryui/1.12.1/jquery-ui.min.js")
+    const jquery = RemoteLibrary("jquery", "file:///usr/share/javascript/jquery/jquery.min.js")
+    const jqueryui = RemoteLibrary("jqueryui", "file:///usr/share/javascript/jquery-ui/jquery-ui.min.js")
     const lunr = RemoteLibrary("lunr", "https://cdnjs.cloudflare.com/ajax/libs/lunr.js/2.3.6/lunr.min.js")
     const lodash = RemoteLibrary("lodash", "https://cdnjs.cloudflare.com/ajax/libs/lodash.js/4.17.15/lodash.min.js")
 
     # headroom
     const headroom_version = "0.10.3"
-    const headroom = RemoteLibrary("headroom", "https://cdnjs.cloudflare.com/ajax/libs/headroom/$(headroom_version)/headroom.min.js")
+    const headroom = RemoteLibrary("headroom", "http://localhost")
     const headroom_jquery = RemoteLibrary(
         "headroom-jquery",
-        "https://cdnjs.cloudflare.com/ajax/libs/headroom/$(headroom_version)/jQuery.headroom.min.js",
+        "http://localhost",
         deps = ["jquery", "headroom"],
     )
 
@@ -404,14 +456,14 @@ module RD
         hljs_version = "9.15.10"
         push!(r, RemoteLibrary(
             "highlight",
-            "https://cdnjs.cloudflare.com/ajax/libs/highlight.js/$(hljs_version)/highlight.min.js"
+            "file:///usr/lib/nodejs/highlight.js/highlight.js"
         ))
         prepend!(languages, ["julia", "julia-repl"])
         for language in languages
             language = jsescape(language)
             push!(r, RemoteLibrary(
                 "highlight-$(language)",
-                "https://cdnjs.cloudflare.com/ajax/libs/highlight.js/$(hljs_version)/languages/$(language).min.js",
+                "file:////usr/lib/nodejs/highlight.js/languages/$(language).js",
                 deps = ["highlight"]
             ))
         end
@@ -428,15 +480,15 @@ module RD
 
     # MathJax & KaTeX
     const katex_version = "0.11.1"
-    const katex_css = "https://cdnjs.cloudflare.com/ajax/libs/KaTeX/$(katex_version)/katex.min.css"
+    const katex_css = "file:///usr/share/javascript/katex/katex.css"
     function mathengine!(r::RequireJS, engine::KaTeX)
         push!(r, RemoteLibrary(
             "katex",
-            "https://cdnjs.cloudflare.com/ajax/libs/KaTeX/$(katex_version)/katex.min.js"
+            "file:///usr/share/javascript/katex/katex.js"
         ))
         push!(r, RemoteLibrary(
             "katex-auto-render",
-            "https://cdnjs.cloudflare.com/ajax/libs/KaTeX/$(katex_version)/contrib/auto-render.min.js",
+            "file:///usr/share/javascript/katex/contrib/auto-render.js",
             deps = ["katex"],
         ))
         push!(r, Snippet(
@@ -452,10 +504,10 @@ module RD
             """
         ))
     end
-    function mathengine!(r::RequireJS, engine::MathJax)
+    function mathengine!(r::RequireJS, engine::MathJax2)
         push!(r, RemoteLibrary(
             "mathjax",
-            "https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.6/MathJax.js?config=TeX-AMS_HTML",
+            "file:///usr/share/javascript/mathjax/MathJax.js?config=TeX-AMS_HTML",
             exports = "MathJax"
         ))
         push!(r, Snippet(["mathjax"], ["MathJax"],
@@ -464,13 +516,27 @@ module RD
             """
         ))
     end
+    function mathengine!(r::RequireJS, engine::MathJax3)
+        push!(r, Snippet([], [],
+            """
+            window.MathJax = $(json_jsescape(engine.config, 2));
+
+            (function () {
+                var script = document.createElement('script');
+                script.src = 'https://cdnjs.cloudflare.com/ajax/libs/mathjax/3.0.5/es5/tex-svg.js';
+                script.async = true;
+                document.head.appendChild(script);
+            })();
+            """
+        ))
+    end
     mathengine(::RequireJS, ::Nothing) = nothing
 end
 
 struct SearchRecord
     src :: String
     page :: Documents.Page
-    loc :: String
+    fragment :: String
     category :: String
     title :: String
     page_title :: String
@@ -496,7 +562,7 @@ end
 
 HTMLContext(doc, settings=HTML()) = HTMLContext(doc, settings, [], "", "", "", [], "", Documents.NavNode("search", "Search", nothing), [])
 
-function SearchRecord(ctx::HTMLContext, navnode; loc="", title=nothing, category="page", text="")
+function SearchRecord(ctx::HTMLContext, navnode; fragment="", title=nothing, category="page", text="")
     page_title = mdflatten(pagetitle(ctx, navnode))
     if title === nothing
         title = page_title
@@ -504,7 +570,7 @@ function SearchRecord(ctx::HTMLContext, navnode; loc="", title=nothing, category
     SearchRecord(
         pretty_url(ctx, get_url(ctx, navnode.page)),
         getpage(ctx, navnode),
-        loc,
+        fragment,
         lowercase(category),
         title,
         page_title,
@@ -515,7 +581,7 @@ end
 function SearchRecord(ctx::HTMLContext, navnode, node::Markdown.Header)
     a = getpage(ctx, navnode).mapping[node]
     SearchRecord(ctx, navnode;
-        loc="$(a.id)-$(a.nth)",
+        fragment=Anchors.fragment(a),
         title=mdflatten(node),
         category="section")
 end
@@ -527,7 +593,7 @@ end
 function JSON.lower(rec::SearchRecord)
     # Replace any backslashes in links, if building the docs on Windows
     src = replace(rec.src, '\\' => '/')
-    ref = string(src, '#', rec.loc)
+    ref = string(src, rec.fragment)
     Dict{String, String}(
         "location" => ref,
         "page" => rec.page_title,
@@ -752,7 +818,7 @@ function render_head(ctx, navnode)
 
         analytics_script(ctx.settings.analytics),
 
-        canonical_link_element(ctx.settings.canonical, src),
+        canonical_link_element(ctx.settings.canonical, pretty_url(ctx, src)),
 
         # Stylesheets.
         map(css_links) do each
@@ -767,11 +833,8 @@ function render_head(ctx, navnode)
 
         script[:src => relhref(src, "siteinfo.js")],
         script[:src => relhref(src, "../versions.js")],
-
-        # Custom user-provided assets.
-        asset_links(src, ctx.settings.assets),
-        # Themes. Note: we reverse the make sure that the default theme (first in the array)
-        # comes as the last <link> tag.
+        # Themes. Note: we reverse the list to make sure that the default theme (first in
+        # the array) comes as the last <link> tag.
         map(Iterators.reverse(enumerate(THEMES))) do (i, theme)
             e = link[".docs-theme-link",
                 :rel => "stylesheet", :type => "text/css",
@@ -782,6 +845,8 @@ function render_head(ctx, navnode)
             return e
         end,
         script[:src => relhref(src, ctx.themeswap_js)],
+        # Custom user-provided assets.
+        asset_links(src, ctx.settings.assets),
     )
 end
 
@@ -1059,7 +1124,22 @@ function render_footer(ctx, navnode)
         link = a[".docs-footer-nextpage", :href => navhref(ctx, navnode.next, navnode)](title, " »")
         push!(navlinks, link)
     end
-    return isempty(navlinks) ? "" : nav[".docs-footer"](navlinks)
+
+    linebreak = div[".flexbox-break"]()
+    footer_content = ctx.settings.footer
+
+    nav_children = []
+    if !isempty(navlinks)
+        push!(nav_children, navlinks, linebreak)
+    end
+
+    if footer_content !== nothing
+        footer_container = domify(ctx, navnode, footer_content)
+        push!(first(footer_container).attributes, :class => "footer-message")
+        push!(nav_children, footer_container)
+    end
+
+    return nav[".docs-footer"](nav_children...)
 end
 
 # Article (page contents)
@@ -1218,16 +1298,18 @@ end
 
 function domify(ctx, navnode, anchor::Anchors.Anchor)
     @tags a
-    aid = "$(anchor.id)-$(anchor.nth)"
+    frag = Anchors.fragment(anchor)
+    legacy = anchor.nth == 1 ? (a[:id => lstrip(frag, '#')*"-1"],) : ()
     if isa(anchor.object, Markdown.Header)
         h = anchor.object
         fixlinks!(ctx, navnode, h)
-        DOM.Tag(Symbol("h$(Utilities.header_level(h))"))[:id => aid](
-            a[".docs-heading-anchor", :href => "#$aid"](mdconvert(h.text, h)),
-            a[".docs-heading-anchor-permalink", :href => "#$aid", :title => "Permalink"]
+        DOM.Tag(Symbol("h$(Utilities.header_level(h))"))[:id => lstrip(frag, '#')](
+            a[".docs-heading-anchor", :href => frag](mdconvert(h.text, h)),
+            legacy...,
+            a[".docs-heading-anchor-permalink", :href => frag, :title => "Permalink"]
         )
     else
-        a[:id => aid, :href => "#$aid"](domify(ctx, navnode, anchor.object))
+        a[:id => frag, :href => frag](legacy..., domify(ctx, navnode, anchor.object))
     end
 end
 
@@ -1264,7 +1346,7 @@ function domify(ctx, navnode, contents::Documents.ContentsNode)
         path = joinpath(navnode_dir, path) # links in ContentsNodes are relative to current page
         path = pretty_url(ctx, relhref(navnode_url, get_url(ctx, path)))
         header = anchor.object
-        url = string(path, '#', anchor.id, '-', anchor.nth)
+        url = string(path, Anchors.fragment(anchor))
         node = a[:href=>url](mdconvert(header.text; droplinks=true))
         level = Utilities.header_level(header)
         push!(lb, level, node)
@@ -1295,7 +1377,7 @@ function domify(ctx, navnode, node::Documents.DocsNode)
 
     # push to search index
     rec = SearchRecord(ctx, navnode;
-        loc=node.anchor.id,
+        fragment=Anchors.fragment(node.anchor),
         title=string(node.object.binding),
         category=Utilities.doccat(node.object),
         text = mdflatten(node.docstr))
@@ -1469,7 +1551,7 @@ function collect_subsections(page::Documents.Page)
                 continue
             end
             anchor = page.mapping[element]
-            push!(sections, (toplevel, "#$(anchor.id)-$(anchor.nth)", element.text))
+            push!(sections, (toplevel, Anchors.fragment(anchor), element.text))
         end
     end
     return sections
@@ -1511,7 +1593,8 @@ mdconvert(b::Markdown.Bold, parent; kwargs...) = Tag(:strong)(mdconvert(b.text,
 
 function mdconvert(c::Markdown.Code, parent::MDBlockContext; kwargs...)
     @tags pre code
-    language = isempty(c.language) ? "none" : c.language
+    language = Utilities.codelang(c.language)
+    language = isempty(language) ? "none" : language
     pre(code[".language-$(language)"](c.code))
 end
 mdconvert(c::Markdown.Code, parent; kwargs...) = Tag(:code)(c.code)
@@ -1595,13 +1678,33 @@ end
 function mdconvert(a::Markdown.Admonition, parent; kwargs...)
     @tags header div
     colorclass =
-        (a.category == "danger")  ? "is-danger"  :
-        (a.category == "warning") ? "is-warning" :
-        (a.category == "note")    ? "is-info"    :
-        (a.category == "info")    ? "is-info"    :
-        (a.category == "tip")     ? "is-success" :
-        (a.category == "compat")  ? "is-compat"  : ""
-    div[".admonition.$(colorclass)"](
+        (a.category == "danger")  ? ".is-danger"  :
+        (a.category == "warning") ? ".is-warning" :
+        (a.category == "note")    ? ".is-info"    :
+        (a.category == "info")    ? ".is-info"    :
+        (a.category == "tip")     ? ".is-success" :
+        (a.category == "compat")  ? ".is-compat"  : begin
+            # If the admonition category is not one of the standard ones, we tag the
+            # admonition div element with a `is-category-$(category)` class. However, we
+            # first carefully sanitize the category name. Strictly speaking, this is not
+            # necessary when were using the Markdown parser in the Julia standard library,
+            # since it restricts the category to [a-z]+. But it is possible for the users to
+            # construct their own Admonition objects with arbitrary category strings and
+            # pass them onto Documenter.
+            #
+            # (1) remove all characters except A-Z, a-z, 0-9 and -
+            cat_sanitized = replace(a.category, r"[^A-Za-z0-9-]" => "")
+            # (2) remove any dashes from the beginning and end of the string
+            cat_sanitized = replace(cat_sanitized, r"^[-]+" => "")
+            cat_sanitized = replace(cat_sanitized, r"[-]+$" => "")
+            # (3) reduce any duplicate dashes in the middle to single dashes
+            cat_sanitized = replace(cat_sanitized, r"[-]+" => "-")
+            cat_sanitized = lowercase(cat_sanitized)
+            # (4) if nothing is left (or the category was empty to begin with), we don't
+            # apply a class
+            isempty(cat_sanitized) ? "" : ".is-category-$(cat_sanitized)"
+        end
+    div[".admonition$(colorclass)"](
         header[".admonition-header"](a.title),
         div[".admonition-body"](mdconvert(a.content, a; kwargs...))
     )
@@ -1630,7 +1733,8 @@ function mdconvert(d::Dict{MIME,Any}, parent; kwargs...)
     elseif haskey(d, MIME"text/markdown"())
         out = Markdown.parse(d[MIME"text/markdown"()])
     elseif haskey(d, MIME"text/plain"())
-        out = Markdown.Code(d[MIME"text/plain"()])
+        @tags pre
+        return pre[".documenter-example-output"](d[MIME"text/plain"()])
     else
         error("this should never happen.")
     end
