@@ -26,7 +26,11 @@ include("TestUtilities.jl"); using .TestUtilities
     include("markdown2.jl")
 
     # DocChecks tests
-    include("docchecks.jl")
+    if haskey(ENV, "DOCUMENTER_TEST_LINKCHECK")
+        include("docchecks.jl")
+    else
+        @info "DOCUMENTER_TEST_LINKCHECK not set, skipping online linkcheck tests."
+    end
 
     # NavNode tests.
     include("navnode.jl")
@@ -46,8 +50,8 @@ include("TestUtilities.jl"); using .TestUtilities
     # MDFlatten tests.
     include("mdflatten.jl")
 
-    # Expanders
-    include("expanders.jl")
+    # Main build pipeline (Builder and Expanders modules)
+    include("pipeline.jl")
 
     # HTMLWriter
     include("htmlwriter.jl")
