@@ -47,6 +47,10 @@ module A
 end
 end
 
+# hasfield was added in Julia 1.2. This definition borrowed from Compat.jl (MIT)
+# Note: this can not be inside the testset
+(VERSION < v"1.2.0-DEV.272") && (hasfield(::Type{T}, name::Symbol) where T = Base.fieldindex(T, name, false) > 0)
+
 @testset "Utilities" begin
     let doc = @doc(length)
         a = Documenter.Utilities.filterdocs(doc, Set{Module}())
@@ -125,6 +129,8 @@ end
         @test Documenter.Utilities.format_line(100:9999, formatting) == "100:9999"
     end
 
+    @test Documenter.Utilities.linerange(Core.svec(), 0) === 0:0
+
     # URL building
     filepath = string(first(methods(Documenter.Utilities.url)).file)
     Sys.iswindows() && (filepath = replace(filepath, "/" => "\\")) # work around JuliaLang/julia#26424
@@ -255,6 +261,7 @@ end
         @test splitby(r"[▶]+", "Ω▶▶Y▶Z▶κ") == ["Ω▶▶", "Y▶", "Z▶", "κ"]
     end
 
+    # This test checks that deprecation warnings are captured correctly
     @static if isdefined(Base, :with_logger)
         @testset "withoutput" begin
             _, _, _, output = Documenter.Utilities.withoutput() do
@@ -263,7 +270,16 @@ end
                 f() = (Base.depwarn("depwarn", :f); nothing)
                 f()
             end
-            @test startswith(output, "println\n[ Info: @info\n┌ Warning: depwarn\n")
+            # The output is dependent on whether the user is running tests with deprecation
+            # warnings enabled or not. To figure out whether that is the case or not, we can
+            # look at the .depwarn field of the undocumented Base.JLOptions object.
+            @test isdefined(Base, :JLOptions)
+            @test hasfield(Base.JLOptions, :depwarn)
+            if Base.JLOptions().depwarn == 0 # --depwarn=no, default on Julia >= 1.5
+                @test output == "println\n[ Info: @info\n"
+            else # --depwarn=yes
+                @test startswith(output, "println\n[ Info: @info\n┌ Warning: depwarn\n")
+            end
         end
     end
 
@@ -426,6 +442,21 @@ end
             @test !occursin("example.com\n/foo", output)
         end
     end
+
+    @testset "codelang" begin
+        @test Documenter.Utilities.codelang("") == ""
+        @test Documenter.Utilities.codelang(" ") == ""
+        @test Documenter.Utilities.codelang("  ") == ""
+        @test Documenter.Utilities.codelang("\t  ") == ""
+        @test Documenter.Utilities.codelang("julia") == "julia"
+        @test Documenter.Utilities.codelang("julia-repl") == "julia-repl"
+        @test Documenter.Utilities.codelang("julia-repl x=y") == "julia-repl"
+        @test Documenter.Utilities.codelang("julia-repl\tx=y") == "julia-repl"
+        @test Documenter.Utilities.codelang(" julia-repl\tx=y") == "julia-repl"
+        @test Documenter.Utilities.codelang("\t julia   \tx=y ") == "julia"
+        @test Documenter.Utilities.codelang("\t julia   \tx=y ") == "julia"
+        @test Documenter.Utilities.codelang("&%^ ***") == "&%^"
+    end
 end
 
 end
